package com.example.nekontakte.posts.api;

import java.util.List;

import com.example.nekontakte.core.configurations.Constants;

import org.apache.coyote.BadRequestException;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/post")
public class PostController {

    private final PostService postService;
    private final UserService userService;
    private final ModelMapper modelMapper;

    public PostController(PostService postService, ModelMapper modelMapper, UserService userService) {
        this.modelMapper = modelMapper;
        this.postService = postService;
        this.userService = userService;
    }

    private PostEntity toEntity(PostDTO dto) {
        return modelMapper.map(dto, PostEntity.class);
    }

    private PostDTO toDTO(PostEntity entity) {
        return modelMapper.map(entity, PostDTO.class);
    }

    @GetMapping
    public List<PostDTO> getAll() {
        return postService.getAll().stream().map(this::toDTO).toList();
    }

    @GetMapping("/user/{userId}")
    public List<PostDTO> getAllByUserId(@PathVariable(name = "userId") Integer userId) {
        return postService.getAllByUser(userService.get(userId)).stream().map(this::toDTO).toList();
    }

    @GetMapping("/{id}")
    public PostDTO get(@PathVariable(name = "id") Integer id) {
        return toDTO(postService.get(id));
    }

    @PostMapping
    public PostDTO create(@RequestBody @Valid PostDTO PostDTO) throws BadRequestException {
        return toDTO(postService.create(toEntity(PostDTO)));
    }

    @PutMapping("/{id}")
    public PostDTO update(@PathVariable(name = "id") Integer id, @RequestBody PostDTO PostDTO) {
        return toDTO(postService.update(id, toEntity(PostDTO)));
    }

    @DeleteMapping("/{id}")
    public PostDTO delete(@PathVariable(name = "id") Integer id) {
        return toDTO(postService.delete(id));
    }

}
