package com.example.nekontakte.posts.service;

import java.util.List;
import java.util.Optional;
import org.springframework.stereotype.Service;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.repository.PostRepository;
import com.example.nekontakte.users.model.UserEntity;

@Service
public class PostService {
    private final PostRepository repository;

    public PostService(PostRepository repository) {
        this.repository = repository;
    }

    public List<PostEntity> getAllByUser(UserEntity user) {
        return repository.getAll().stream().filter(item -> item.getUser().equals(user)).toList();
    }

    public List<PostEntity> getAll() {
        return repository.getAll();
    }

    public PostEntity get(Integer id) {
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public PostEntity create(PostEntity entity) throws org.apache.coyote.BadRequestException {
        if (entity.getImage() == null && entity.getHtml() == null) {
            throw new org.apache.coyote.BadRequestException("Image or Html must be not null");
        }
        return repository.create(entity);
    }

    public PostEntity update(Integer id, PostEntity entity) {
        final PostEntity existEntity = get(id);
        existEntity.setUser(entity.getUser());
        existEntity.setPubDate(entity.getPubDate());
        existEntity.setImage(entity.getImage());
        existEntity.setHtml(entity.getHtml());
        return repository.update(existEntity);
    }

    public PostEntity delete(Integer id) {
        final PostEntity existsentity = get(id);
        return repository.delete(existsentity);
    }

}
