package com.example.nekontakte.subscribes.api;

import java.util.List;

import com.example.nekontakte.core.configurations.Constants;
import com.example.nekontakte.posts.api.PostDTO;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.nekontakte.subscribes.model.SubscribeEntity;
import com.example.nekontakte.subscribes.service.SubscribeService;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/subscribe")
public class SubscribeController {
    private final SubscribeService subscribeService;
    private final UserService userService;
    private final ModelMapper modelMapper;

    public SubscribeController(SubscribeService subscribeService, ModelMapper modelMapper, UserService userService) {
        this.modelMapper = modelMapper;
        this.subscribeService = subscribeService;
        this.userService = userService;
    }

    private SubscribeEntity toEntity(SubscribeDTO dto) {
        return modelMapper.map(dto, SubscribeEntity.class);
    }

    private SubscribeDTO toDTO(SubscribeEntity entity) {
        return modelMapper.map(entity, SubscribeDTO.class);
    }

    @GetMapping
    public List<SubscribeDTO> getAll() {
        return subscribeService.getAll().stream().map(this::toDTO).toList();
    }

    @GetMapping("/user/{userId}")
    public List<SubscribeDTO> getAllByUserId(@PathVariable(name = "userId") Integer userId) {
        return subscribeService.getAllByUser(userService.get(userId)).stream().map(this::toDTO).toList();
    }

    @GetMapping("/{id}")
    public SubscribeDTO get(@PathVariable(name = "id") Integer id) {
        return toDTO(subscribeService.get(id));
    }

    @PostMapping
    public SubscribeDTO create(@RequestBody @Valid SubscribeDTO SubscribeDTO) {
        return toDTO(subscribeService.create(toEntity(SubscribeDTO)));
    }

    @PutMapping("/{id}")
    public SubscribeDTO update(@PathVariable(name = "id") Integer id, @RequestBody SubscribeDTO SubscribeDTO) {
        return toDTO(subscribeService.update(id, toEntity(SubscribeDTO)));
    }

    @DeleteMapping("/{id}")
    public SubscribeDTO delete(@PathVariable(name = "id") Integer id) {
        return toDTO(subscribeService.delete(id));
    }
}
