package com.example.nekontakte.subscribes.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.subscribes.model.SubscribeEntity;
import com.example.nekontakte.subscribes.repository.SubscribeRepository;
import com.example.nekontakte.users.model.UserEntity;

@Service
public class SubscribeService {
    private final SubscribeRepository repository;

    public SubscribeService(SubscribeRepository repository) {
        this.repository = repository;
    }

    public List<SubscribeEntity> getAll() {
        return repository.getAll();
    }

    public List<SubscribeEntity> getAllByUser(UserEntity user) {
        return repository.getAll().stream().filter(item -> item.getUser().equals(user)).toList();
    }

    public SubscribeEntity get(Integer id) {
        return Optional.ofNullable(repository.get(id)).orElseThrow(() -> new NotFoundException(id));
    }

    public SubscribeEntity create(SubscribeEntity entity) {
        return repository.create(entity);
    }

    public SubscribeEntity update(Integer id, SubscribeEntity entity) {
        final SubscribeEntity existEntity = get(id);
        existEntity.setId(entity.getId());
        existEntity.setSubscriber(entity.getSubscriber());
        existEntity.setUser(entity.getUser());
        return repository.update(existEntity);
    }

    public SubscribeEntity delete(Integer id) {
        final SubscribeEntity existsentity = get(id);
        return repository.delete(existsentity);
    }
}
