package com.example.nekontakte;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.apache.coyote.BadRequestException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.model.UserEntity;

import org.junit.jupiter.api.TestMethodOrder;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class PostServiceTests {
    @Autowired
    private PostService postService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> postService.get(0));
    }

    @Test
    @Order(1)
    void createTest() throws ParseException, BadRequestException {
        SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);
        SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("dd.MM.yyyy, HH:mm:ss", Locale.ENGLISH);

        final UserEntity firstUser = new UserEntity(
                null,
                "nspotapov",
                "pass123456",
                true,
                "Никита",
                "Потапов",
                dateFormatter.parse("17.02.2003"),
                "Ульяновск",
                null,
                "Я здесь админ");
        final UserEntity secondUser = new UserEntity(
                null,
                "ekallin",
                "pass87654321",
                false,
                "Елена",
                "Каллин",
                dateFormatter.parse("14.06.2005"),
                "Новоульяновск",
                null,
                null);

        final PostEntity firstPost = postService
                .create(new PostEntity(
                        null,
                        firstUser,
                        dateTimeFormatter.parse("01.04.2024, 17:02:34"),
                        null, "Первый тестовый пост"));

        postService.create(new PostEntity(
                null,
                firstUser,
                dateTimeFormatter.parse("22.03.2024, 09:34:01"),
                "myimage.jpg", "Второй тестовый пост"));

        postService.create(new PostEntity(
                null,
                secondUser,
                dateTimeFormatter.parse("13.02.2024, 18:01:01"),
                "ohmypost.jpg", "Первый постовый пост"));

        Assertions.assertThrows(BadRequestException.class, () -> postService
                .create(new PostEntity(
                        null,
                        secondUser,
                        dateTimeFormatter.parse("13.02.2024, 18:01:01"),
                        null, null)));

        Assertions.assertEquals(firstPost, postService.get(1));
        Assertions.assertEquals(firstUser, postService.get(1).getUser());
        Assertions.assertEquals(secondUser, postService.get(3).getUser());
        Assertions.assertEquals(3, postService.getAll().size());
        Assertions.assertEquals(2, postService.getAllByUser(firstUser).size());
    }

    @Test
    @Order(2)
    void update() throws BadRequestException, ParseException {
        SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);
        SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("dd.MM.yyyy, HH:mm:ss", Locale.ENGLISH);

        final UserEntity firstUser = new UserEntity(
                null,
                "nspotapov",
                "pass123456",
                true,
                "Никита",
                "Потапов",
                dateFormatter.parse("17.02.2003"),
                "Ульяновск",
                null,
                "Я здесь админ");

        postService.create(new PostEntity(
                null,
                firstUser,
                dateTimeFormatter.parse("01.04.2024, 17:02:34"),
                null, "Первый тестовый пост"));

        Integer postId = 1;
        PostEntity oldPost = postService.get(postId);
        String oldImage = oldPost.getImage();
        PostEntity newPost = new PostEntity(
                oldPost.getId(),
                oldPost.getUser(),
                oldPost.getPubDate(),
                "new image",
                oldPost.getHtml());
        postService.update(postId, newPost);
        Assertions.assertNotEquals(
                postService.get(postId).getImage(), oldImage);
    }

    @Test
    @Order(3)
    void delete() {
        final int oldCount = postService.getAll().size();
        postService.delete(1);
        Assertions.assertEquals(oldCount - 1, postService.getAll().size());
    }
}
