package com.example.nekontakte;

import org.junit.jupiter.api.TestMethodOrder;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.users.service.UserService;
import com.example.nekontakte.users.model.UserEntity;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class UserServiceTests {

	@Autowired
	private UserService userService;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> userService.get(0));
	}

	@Test
	@Order(1)
	void createTest() throws ParseException {
		SimpleDateFormat dateFormatter = new SimpleDateFormat("dd.MM.yyyy", Locale.ENGLISH);
		final UserEntity firstUser = userService.create(new UserEntity(
				null,
				"nspotapov",
				"pass123456",
				true,
				"Никита",
				"Потапов",
				dateFormatter.parse("17.02.2003"),
				"Ульяновск",
				null,
				"Я здесь админ"));
		userService.create(new UserEntity(
				null,
				"ekallin",
				"pass87654321",
				false,
				"Елена",
				"Каллин",
				dateFormatter.parse("14.06.2005"),
				"Новоульяновск",
				null,
				null));
		userService.create(new UserEntity(
				null,
				"olegulya",
				"passOleg",
				false,
				"Олег",
				"Тинькофф",
				dateFormatter.parse("25.12.1967"),
				"Полысаево",
				null,
				"Вчера я потерял $250 млн за день, были дни и похуже, миллиарды в день терял. Поэтому это позитивный очень день"));
		Assertions.assertEquals(firstUser, userService.get(1));
		Assertions.assertEquals(3, userService.getAll().size());
	}

	@Test
	@Order(2)
	void update() {
		final String newPassword = "newpassword";
		final UserEntity existEntity = userService.get(1);
		final String oldPassword = existEntity.getPassword();
		final UserEntity entity = new UserEntity(
				null,
				existEntity.getUsername(),
				newPassword,
				existEntity.getIsAdmin(),
				existEntity.getName(),
				existEntity.getSurname(),
				existEntity.getBirthday(),
				existEntity.getCity(),
				existEntity.getAvatarImg(),
				existEntity.getStatus());
		final UserEntity newEntity = userService.update(1, entity);
		Assertions.assertEquals(newPassword, newEntity.getPassword());
		Assertions.assertNotEquals(oldPassword, newEntity.getPassword());
	}

	@Test
	@Order(3)
	void delete() {
		userService.delete(1);
		Assertions.assertEquals(2, userService.getAll().size());
	}
}
